@@ -95,6 +95,9 @@ class Variable(object):
 
     def isTempVariable(self):
         return False
+
+    def isLocalsDictVariable(self):
+        return False
     # pylint: enable=R0201
 
     def addVariableUser(self, user):
@@ -310,6 +313,26 @@ class TempVariable(Variable):
         return True
 
 
+class LocalsDictVariable(Variable):
+    __slots__ = ()
+
+    def __init__(self, owner, variable_name):
+        Variable.__init__(
+            self,
+            owner         = owner,
+            variable_name = variable_name
+        )
+
+    def __repr__(self):
+        return "<LocalsDictVariable '%s' of '%s'>" % (
+            self.getName(),
+            self.getOwner().getName()
+        )
+
+    def isLocalsDictVariable(self):
+        return True
+
+
 def updateVariablesFromCollection(old_collection, new_collection):
     # After removing/adding traces, we need to pre-compute the users state
     # information.
