@@ -51,7 +51,8 @@ from .BuiltinCodes import (
     generateBuiltinBytearray1Code,
     generateBuiltinBytearray3Code,
     generateBuiltinClassmethodCode,
-    generateBuiltinComplexCode,
+    generateBuiltinComplex1Code,
+    generateBuiltinComplex2Code,
     generateBuiltinFloatCode,
     generateBuiltinHexCode,
     generateBuiltinOctCode,
@@ -169,6 +170,7 @@ from .IntegerCodes import (
 from .IteratorCodes import (
     generateBuiltinIter1Code,
     generateBuiltinIter2Code,
+    generateBuiltinIterForUnpackCode,
     generateBuiltinLenCode,
     generateBuiltinNext1Code,
     generateBuiltinNext2Code,
@@ -239,7 +241,8 @@ from .SliceCodes import (
 )
 from .StringCodes import (
     generateBuiltinAsciiCode,
-    generateBuiltinBytesCode,
+    generateBuiltinBytes1Code,
+    generateBuiltinBytes3Code,
     generateBuiltinChrCode,
     generateBuiltinFormatCode,
     generateBuiltinOrdCode,
@@ -537,6 +540,7 @@ setExpressionDispatchDict(
         "EXPRESSION_BUILTIN_EXECFILE"                 : generateExecfileCode,
         "EXPRESSION_BUILTIN_EVAL"                     : generateEvalCode,
         "EXPRESSION_BUILTIN_EXEC"                     : generateEvalCode,
+        "EXPRESSION_BUILTIN_ITER_FOR_UNPACK"          : generateBuiltinIterForUnpackCode,
         "EXPRESSION_BUILTIN_ITER1"                    : generateBuiltinIter1Code,
         "EXPRESSION_BUILTIN_ITER2"                    : generateBuiltinIter2Code,
         "EXPRESSION_BUILTIN_NEXT1"                    : generateBuiltinNext1Code,
@@ -554,10 +558,12 @@ setExpressionDispatchDict(
         "EXPRESSION_BUILTIN_LONG1"                    : generateBuiltinLong1Code,
         "EXPRESSION_BUILTIN_LONG2"                    : generateBuiltinLong2Code,
         "EXPRESSION_BUILTIN_FLOAT"                    : generateBuiltinFloatCode,
-        "EXPRESSION_BUILTIN_COMPLEX"                  : generateBuiltinComplexCode,
+        "EXPRESSION_BUILTIN_COMPLEX1"                 : generateBuiltinComplex1Code,
+        "EXPRESSION_BUILTIN_COMPLEX2"                 : generateBuiltinComplex2Code,
         "EXPRESSION_BUILTIN_LEN"                      : generateBuiltinLenCode,
         "EXPRESSION_BUILTIN_STR"                      : generateBuiltinStrCode,
-        "EXPRESSION_BUILTIN_BYTES"                    : generateBuiltinBytesCode,
+        "EXPRESSION_BUILTIN_BYTES1"                   : generateBuiltinBytes1Code,
+        "EXPRESSION_BUILTIN_BYTES3"                   : generateBuiltinBytes3Code,
         "EXPRESSION_BUILTIN_UNICODE"                  : generateBuiltinUnicodeCode,
         "EXPRESSION_BUILTIN_CHR"                      : generateBuiltinChrCode,
         "EXPRESSION_BUILTIN_ORD"                      : generateBuiltinOrdCode,
