@@ -22,7 +22,6 @@ whose implementation lives here. The creation itself also lives here.
 
 """
 
-from .Checkers import checkStatementsSequenceOrNone
 from .ExpressionBases import ExpressionChildrenHavingBase
 from .FunctionNodes import ExpressionFunctionEntryPointBase
 
@@ -68,11 +67,11 @@ class ExpressionMakeCoroutineObject(ExpressionChildrenHavingBase):
         self.variable_closure_traces = []
 
         for closure_variable in self.getCoroutineRef().getFunctionBody().getClosureVariables():
-            version, trace = trace_collection.getVariableCurrentTraceVersion(closure_variable)
+            trace = trace_collection.getVariableCurrentTrace(closure_variable)
             trace.addClosureUsage()
 
             self.variable_closure_traces.append(
-                (closure_variable, version, trace)
+                (closure_variable, trace)
             )
 
         # TODO: Coroutine body may know something too.
@@ -91,15 +90,6 @@ class ExpressionMakeCoroutineObject(ExpressionChildrenHavingBase):
 class ExpressionCoroutineObjectBody(ExpressionFunctionEntryPointBase):
     kind = "EXPRESSION_COROUTINE_OBJECT_BODY"
 
-    named_children = (
-        "body",
-    )
-
-    checkers = {
-        # TODO: Is "None" really an allowed value.
-        "body" : checkStatementsSequenceOrNone
-    }
-
     qualname_setup = None
 
     def __init__(self, provider, name, flags, source_ref):
