@@ -31,6 +31,11 @@ classes.
 
 from nuitka import Options, Variables
 from nuitka.PythonVersions import python_version
+from nuitka.specs.ParameterSpecs import (
+    ParameterSpec,
+    TooManyArguments,
+    matchCall
+)
 from nuitka.tree.Extractions import updateVariableUsage
 
 from .Checkers import checkStatementsSequenceOrNone
@@ -38,6 +43,7 @@ from .CodeObjectSpecs import CodeObjectSpec
 from .ExpressionBases import (
     CompileTimeConstantExpressionBase,
     ExpressionBase,
+    ExpressionChildHavingBase,
     ExpressionChildrenHavingBase
 )
 from .FutureSpecs import fromFlags
@@ -47,7 +53,6 @@ from .IndicatorMixins import (
 )
 from .LocalsScopes import LocalsDictHandle
 from .NodeBases import (
-    ChildrenHavingMixin,
     ClosureGiverNodeMixin,
     ClosureTakerMixin,
     SideEffectsFromChildrenMixin
@@ -57,7 +62,6 @@ from .NodeMakingHelpers import (
     makeRaiseExceptionReplacementExpressionFromInstance,
     wrapExpressionWithSideEffects
 )
-from .ParameterSpecs import ParameterSpec, TooManyArguments, matchCall
 
 
 class MaybeLocalVariableUsage(Exception):
@@ -65,17 +69,19 @@ class MaybeLocalVariableUsage(Exception):
 
 
 class ExpressionFunctionBodyBase(ClosureTakerMixin, ClosureGiverNodeMixin,
-                                 ExpressionChildrenHavingBase):
+                                 ExpressionChildHavingBase):
+
+    named_child = "body"
+
+    checker = checkStatementsSequenceOrNone
 
     def __init__(self, provider, name, code_prefix, flags, source_ref, body):
         while provider.isExpressionOutlineBody():
             provider = provider.getParentVariableProvider()
 
-        ExpressionChildrenHavingBase.__init__(
+        ExpressionChildHavingBase.__init__(
             self,
-            values     = {
-                "body" : body # Might be None initially in some cases.
-            },
+            value      = body, # Might be None initially in some cases.
             source_ref = source_ref
         )
 
@@ -106,7 +112,6 @@ class ExpressionFunctionBodyBase(ClosureTakerMixin, ClosureGiverNodeMixin,
         # Non-local declarations.
         self.non_local_declarations = []
 
-
     @staticmethod
     def isExpressionFunctionBodyBase():
         return True
@@ -350,6 +355,9 @@ class ExpressionFunctionBodyBase(ClosureTakerMixin, ClosureGiverNodeMixin,
         else:
             return self.getBody().mayRaiseException(exception_type)
 
+    getBody = ExpressionChildHavingBase.childGetter("body")
+    setBody = ExpressionChildHavingBase.childSetter("body")
+
 
 class ExpressionFunctionEntryPointBase(EntryPointMixin, ExpressionFunctionBodyBase):
     def __init__(self, provider, name, code_prefix, flags, source_ref):
@@ -376,9 +384,6 @@ class ExpressionFunctionEntryPointBase(EntryPointMixin, ExpressionFunctionBodyBa
         else:
             self.locals_scope = None
 
-    getBody = ChildrenHavingMixin.childGetter("body")
-    setBody = ChildrenHavingMixin.childSetter("body")
-
     def computeFunctionRaw(self, trace_collection):
         from nuitka.optimizations.TraceCollections import \
             TraceCollectionFunction
@@ -412,9 +417,6 @@ class ExpressionFunctionEntryPointBase(EntryPointMixin, ExpressionFunctionBodyBa
 
 class ExpressionFunctionBody(MarkUnoptimizedFunctionIndicatorMixin,
                              ExpressionFunctionEntryPointBase):
-    # We really want these many ancestors, as per design, we add properties via
-    # base class mix-ins a lot, leading to many methods, pylint: disable=R0901
-
     kind = "EXPRESSION_FUNCTION_BODY"
 
     named_children = (
@@ -681,11 +683,11 @@ class ExpressionFunctionCreation(SideEffectsFromChildrenMixin,
         self.variable_closure_traces = []
 
         for closure_variable in self.getFunctionRef().getFunctionBody().getClosureVariables():
-            version, trace = trace_collection.getVariableCurrentTraceVersion(closure_variable)
+            trace = trace_collection.getVariableCurrentTrace(closure_variable)
             trace.addClosureUsage()
 
             self.variable_closure_traces.append(
-                (closure_variable, version, trace)
+                (closure_variable, trace)
             )
 
         # TODO: Function body may know something too.
@@ -751,6 +753,7 @@ class ExpressionFunctionCreation(SideEffectsFromChildrenMixin,
                 star_list_arg = call_spec.getStarListArgumentName(),
                 star_dict_arg = call_spec.getStarDictArgumentName(),
                 num_defaults  = call_spec.getDefaultCount(),
+                num_posonly   = call_spec.getPositionalOnlyCount(),
                 positional    = args_tuple,
                 pairs         = ()
             )
@@ -955,11 +958,11 @@ class ExpressionFunctionCall(ExpressionChildrenHavingBase):
         self.variable_closure_traces = []
 
         for closure_variable in function_body.getClosureVariables():
-            version, trace = trace_collection.getVariableCurrentTraceVersion(closure_variable)
+            trace = trace_collection.getVariableCurrentTrace(closure_variable)
             trace.addClosureUsage()
 
             self.variable_closure_traces.append(
-                (closure_variable, version, trace)
+                (closure_variable, trace)
             )
 
         return self, None, None
