@@ -22,17 +22,14 @@ fallback can be optimized to no fallback variants.
 
 """
 
-
-
 from nuitka import Variables
 from nuitka.optimizations.TraceCollections import TraceCollectionBranch
 from nuitka.PythonVersions import python_version
 
-from .CallNodes import ExpressionCallEmpty
 from .ConditionalNodes import ExpressionConditional
 from .ConstantRefNodes import ExpressionConstantDictEmptyRef
 from .ExpressionBases import ExpressionBase, ExpressionChildrenHavingBase
-from .NodeBases import NodeBase, StatementChildrenHavingBase
+from .NodeBases import StatementBase, StatementChildHavingBase
 
 
 class ExpressionLocalsVariableRefORFallback(ExpressionChildrenHavingBase):
@@ -40,11 +37,11 @@ class ExpressionLocalsVariableRefORFallback(ExpressionChildrenHavingBase):
 
     named_children = ("fallback",)
 
-    def __init__(self, locals_scope, variable_name, fallback_node, source_ref):
+    def __init__(self, locals_scope, variable_name, fallback, source_ref):
         ExpressionChildrenHavingBase.__init__(
             self,
             values     = {
-                "fallback" : fallback_node
+                "fallback" : fallback
             },
             source_ref = source_ref
         )
@@ -52,27 +49,62 @@ class ExpressionLocalsVariableRefORFallback(ExpressionChildrenHavingBase):
         assert locals_scope is not None
 
         self.locals_scope = locals_scope
-        self.variable_name = variable_name
+        self.variable = locals_scope.getLocalsDictVariable(variable_name)
+
+        self.variable_trace = None
 
     def getDetails(self):
         return {
-            "variable_name" : self.variable_name,
+            "locals_scope"  : self.locals_scope,
+            "variable_name" : self.getVariableName(),
         }
 
     def getVariableName(self):
-        return self.variable_name
+        return self.variable.getName()
 
     def getLocalsDictScope(self):
         return self.locals_scope
 
-
     def computeExpressionRaw(self, trace_collection):
-        # TODO: Use dictionary tracings for locals dict and then cut
-        # to no branch variant if possible.
+        self.variable_trace = trace_collection.getVariableCurrentTrace(
+            variable = self.variable
+        )
 
-        possible = trace_collection.onLocalsDictGet(self.variable_name)
+        replacement = self.variable_trace.getReplacementNode(self)
+
+        if replacement is not None:
+            trace_collection.signalChange(
+                "new_expression",
+                self.source_ref,
+                "Value propagated for '%s' from '%s'." % (
+                    self.variable.getName(),
+                    replacement.getSourceReference().getAsString()
+                )
+            )
+
+            # Need to compute the replacement still.
+            return replacement.computeExpressionRaw(trace_collection)
+
+        # If we cannot be sure if the value is set, then we need the fallback,
+        # otherwise we could remove it simply.
+        if self.variable_trace.mustHaveValue():
+            trace_collection.signalChange(
+                "new_expression",
+                self.source_ref,
+                "Name '%s' must be in locals dict." % self.variable.getName()
+            )
+
+            result = ExpressionLocalsVariableRef(
+                locals_scope  = self.locals_scope,
+                variable_name = self.variable.getName(),
+                source_ref    = self.source_ref
+            )
+
+            # Need to compute the replacement still.
+            return result.computeExpressionRaw(trace_collection)
+        else:
+            trace_collection.onExceptionRaiseExit(BaseException)
 
-        if possible is not True:
             branch_fallback = TraceCollectionBranch(
                 parent = trace_collection,
                 name   = "fallback node usage"
@@ -85,8 +117,13 @@ class ExpressionLocalsVariableRefORFallback(ExpressionChildrenHavingBase):
                 None
             )
 
-        if possible is False:
-            return self.subnode_fallback, "new_expression", "Name '%s' cannot be in locals dict." % self.variable_name
+        # if we can be sure if doesn't have a value set, go to the fallback directly.
+        if self.variable_trace.mustNotHaveValue():
+            return (
+                self.subnode_fallback,
+                "new_expression",
+                "Name '%s' cannot be in locals dict." % self.variable.getName()
+            )
         else:
             return self, None, None
 
@@ -97,7 +134,7 @@ class ExpressionLocalsVariableRefORFallback(ExpressionChildrenHavingBase):
         trace_collection.onControlFlowEscape(self)
 
         if not Variables.complete and \
-           self.variable_name in ("dir", "eval", "exec", "execfile", "locals", "vars") and \
+           self.variable.getName() in ("dir", "eval", "exec", "execfile", "locals", "vars") and \
            self.subnode_fallback.isExpressionVariableRef() and \
            self.subnode_fallback.getVariable().isModuleVariable():
             # Just inform the collection that all escaped.
@@ -105,35 +142,41 @@ class ExpressionLocalsVariableRefORFallback(ExpressionChildrenHavingBase):
 
         if self.subnode_fallback.isExpressionBuiltinRef() or \
            self.subnode_fallback.isExpressionConstantTypeRef():
-            if call_node.isExpressionCallEmpty():
-                result = ExpressionConditional(
-                    condition      = ExpressionLocalsVariableCheck(
-                        locals_scope  = self.locals_scope,
-                        variable_name = self.variable_name,
-                        source_ref    = self.source_ref
+            variable_name = self.variable.getName()
 
-                    ),
-                    expression_yes = ExpressionCallEmpty(
-                        called     = ExpressionLocalsVariableRef(
-                            locals_scope  = self.locals_scope,
-                            variable_name = self.variable_name,
-                            source_ref    = self.source_ref
-                        ),
-                        source_ref = call_node.source_ref
-                    ),
-                    expression_no  = ExpressionCallEmpty(
-                        called     = self.subnode_fallback,
-                        source_ref = call_node.source_ref
-                    ),
-                    source_ref     = self.source_ref
+            # Create a cloned node with the locals variable.
+            call_node_clone = call_node.makeClone()
+            call_node_clone.setCalled(
+                ExpressionLocalsVariableRef(
+                    locals_scope  = self.locals_scope,
+                    variable_name = variable_name,
+                    source_ref    = self.source_ref
                 )
+            )
 
-                return result, "new_expression", "Moved empty call of uncertain dict variable to inside."
+            # Make the original one for the fallback
+            call_node = call_node.makeClone()
+            call_node.setCalled(self.subnode_fallback)
+
+            result = ExpressionConditional(
+                condition      = ExpressionLocalsVariableCheck(
+                    locals_scope  = self.locals_scope,
+                    variable_name = variable_name,
+                    source_ref    = self.source_ref
+
+                ),
+                expression_yes = call_node_clone,
+                expression_no  = call_node,
+                source_ref     = self.source_ref
+            )
+
+            return result, "new_expression", "Moved call of uncertain dict variable to inside."
 
         return call_node, None, None
 
     def mayRaiseException(self, exception_type):
-        return python_version >= 300 or self.subnode_fallback.mayRaiseException(exception_type)
+        return python_version >= 300 or \
+               self.subnode_fallback.mayRaiseException(exception_type)
 
 
 class ExpressionLocalsMappingVariableRefORFallback(ExpressionLocalsVariableRefORFallback):
@@ -143,7 +186,7 @@ class ExpressionLocalsMappingVariableRefORFallback(ExpressionLocalsVariableRefOR
 class ExpressionLocalsVariableRef(ExpressionBase):
     kind = "EXPRESSION_LOCALS_VARIABLE_REF"
 
-    __slots__ = "variable_name", "locals_scope"
+    __slots__ = "variable", "locals_scope", "variable_trace"
 
     def __init__(self, locals_scope, variable_name, source_ref):
 
@@ -153,43 +196,63 @@ class ExpressionLocalsVariableRef(ExpressionBase):
         )
 
         self.locals_scope = locals_scope
-        self.variable_name = variable_name
+        self.variable = locals_scope.getLocalsDictVariable(variable_name)
+
+        self.variable_trace = None
 
     def getDetails(self):
         return {
-            "variable_name" : self.variable_name,
+            "variable_name" : self.getVariableName(),
             "locals_scope"  : self.locals_scope
         }
 
     def getDetailsForDisplay(self):
         return {
-            "variable_name" : self.variable_name,
+            "variable_name" : self.getVariableName(),
         }
 
     def getVariableName(self):
-        return self.variable_name
+        return self.variable.getName()
 
     def getLocalsDictScope(self):
         return self.locals_scope
 
     def computeExpressionRaw(self, trace_collection):
-        # TODO: Use dictionary tracings for locals dict and then cut
-        # to no branch variant if possible.
+        self.variable_trace = trace_collection.getVariableCurrentTrace(
+            variable = self.variable
+        )
 
-        _possible = trace_collection.onLocalsDictGet(self.variable_name)
+        replacement = self.variable_trace.getReplacementNode(self)
+
+        if replacement is not None:
+            trace_collection.signalChange(
+                "new_expression",
+                self.source_ref,
+                "Value propagated for '%s' from '%s'." % (
+                    self.variable.getName(),
+                    replacement.getSourceReference().getAsString()
+                )
+            )
+
+            # Need to compute the replacement still.
+            return replacement.computeExpressionRaw(trace_collection)
+
+        if not self.variable_trace.mustHaveValue():
+            trace_collection.onExceptionRaiseExit(BaseException)
 
         return self, None, None
 
+
     def computeExpressionCall(self, call_node, call_args, call_kw,
                               trace_collection):
         trace_collection.onExceptionRaiseExit(BaseException)
 
         trace_collection.onControlFlowEscape(self)
-
         return call_node, None, None
 
     def mayRaiseException(self, exception_type):
-        return False
+        return self.variable_trace is None or \
+               not self.variable_trace.mustHaveValue()
 
 
 class ExpressionLocalsVariableCheck(ExpressionBase):
@@ -209,6 +272,7 @@ class ExpressionLocalsVariableCheck(ExpressionBase):
 
     def getDetails(self):
         return {
+            "locals_scope"  : self.locals_scope,
             "variable_name" : self.variable_name,
         }
 
@@ -222,40 +286,45 @@ class ExpressionLocalsVariableCheck(ExpressionBase):
         return self, None, None
 
 
-class StatementLocalsDictOperationSet(StatementChildrenHavingBase):
+class StatementLocalsDictOperationSet(StatementChildHavingBase):
     kind = "STATEMENT_LOCALS_DICT_OPERATION_SET"
 
-    named_children = (
-        "value",
-    )
+    named_child = "value"
+
+    __slots__ = ("variable", "variable_version", "variable_trace", "locals_scope")
+
+    # TODO: Specialize for Python3 maybe to save attribute for Python2.
+    may_raise_set = python_version >= 300
 
     def __init__(self, locals_scope, variable_name, value, source_ref):
         assert type(variable_name) is str
         assert value is not None
 
-        StatementChildrenHavingBase.__init__(
+        StatementChildHavingBase.__init__(
             self,
-            values     = {
-                "value" : value
-            },
+            value      = value,
             source_ref = source_ref
         )
 
         assert locals_scope is not None
 
+        self.variable = locals_scope.getLocalsDictVariable(
+            variable_name = variable_name
+        )
+
+        self.variable_version = self.variable.allocateTargetNumber()
+        self.variable_trace = None
+
         self.locals_scope = locals_scope
-        self.variable_name = variable_name
 
-        # TODO: Specialize for Python3 maybe to save attribute for Python2.
-        self.may_raise_set = python_version >= 300
 
     def getDetails(self):
         return {
-            "variable_name" : self.variable_name
+            "variable_name" : self.getVariableName()
         }
 
     def getVariableName(self):
-        return self.variable_name
+        return self.variable.getName()
 
     def getLocalsDictScope(self):
         return self.locals_scope
@@ -268,26 +337,37 @@ class StatementLocalsDictOperationSet(StatementChildrenHavingBase):
         if result is not self:
             return result, change_tags, change_desc
 
-        self.may_raise_set = trace_collection.onLocalsDictSet(
-            variable_name = self.variable_name,
-            value         = self.subnode_value
+        self.variable_trace = trace_collection.onVariableSet(
+            variable    = self.variable,
+            version     = self.variable_version,
+            assign_node = self
         )
 
+        if self.may_raise_set:
+            trace_collection.onExceptionRaiseExit(BaseException)
+
         return self, None, None
 
     def mayRaiseException(self, exception_type):
         return self.may_raise_set or self.subnode_value.mayRaiseException(exception_type)
 
+    def getStatementNiceName(self):
+        return "locals dictionary value set statement"
 
-class StatementLocalsDictOperationDel(NodeBase):
+
+class StatementLocalsDictOperationDel(StatementBase):
     kind = "STATEMENT_LOCALS_DICT_OPERATION_DEL"
 
-    __slots__ = "variable_name", "may_raise_del", "locals_scope"
+    __slots__ = "variable", "variable_version", "previous_trace", "locals_scope"
+
+    # TODO: Specialize for Python3 maybe to save attribute for Python2.
+    may_raise_del = python_version >= 300
+
 
     def __init__(self, locals_scope, variable_name, source_ref):
         assert type(variable_name) is str
 
-        NodeBase.__init__(
+        StatementBase.__init__(
             self,
             source_ref = source_ref
         )
@@ -295,47 +375,66 @@ class StatementLocalsDictOperationDel(NodeBase):
         assert locals_scope is not None
 
         self.locals_scope = locals_scope
-        self.variable_name = variable_name
 
-        # TODO: Specialize for Python3 maybe to save attribute for Python2.
-        self.may_raise_del = python_version >= 300
+        self.variable = locals_scope.getLocalsDictVariable(variable_name)
+        self.variable_version = self.variable.allocateTargetNumber()
+
+        self.previous_trace = None
 
     def getDetails(self):
         return {
-            "variable_name" : self.variable_name,
+            "variable_name" : self.getVariableName(),
             "locals_scope"  : self.locals_scope
         }
 
     def getVariableName(self):
-        return self.variable_name
+        return self.variable.getName()
 
     def getLocalsDictScope(self):
         return self.locals_scope
 
     def computeStatement(self, trace_collection):
-        self.may_raise_del = trace_collection.onLocalsDictDel(
-            variable_name = self.variable_name
+        self.previous_trace = trace_collection.getVariableCurrentTrace(self.variable)
+
+        # The "del" is a potential use of a value. TODO: This could be made more
+        # beautiful indication, as it's not any kind of usage.
+        self.previous_trace.addPotentialUsage()
+
+        # We may not exception exit now during the __del__ unless there is no value.
+        if not self.previous_trace.mustHaveValue():
+            trace_collection.onExceptionRaiseExit(BaseException)
+
+        # Record the deletion, needs to start a new version then.
+        _variable_trace = trace_collection.onVariableDel(
+            variable = self.variable,
+            version  = self.variable_version
         )
 
+        trace_collection.onVariableContentEscapes(self.variable)
+
+        # Any code could be run, note that.
+        trace_collection.onControlFlowEscape(self)
+
         return self, None, None
 
     def mayRaiseException(self, exception_type):
         return self.may_raise_del
 
+    def getStatementNiceName(self):
+        return "locals dictionary value del statement"
 
-class StatementSetLocals(StatementChildrenHavingBase):
+
+class StatementSetLocals(StatementChildHavingBase):
     kind = "STATEMENT_SET_LOCALS"
 
-    named_children = (
-        "new_locals",
-    )
+    named_child = "new_locals"
+
+    __slots__ = ("locals_scope",)
 
     def __init__(self, locals_scope, new_locals, source_ref):
-        StatementChildrenHavingBase.__init__(
+        StatementChildHavingBase.__init__(
             self,
-            values     = {
-                "new_locals" : new_locals,
-            },
+            value      = new_locals,
             source_ref = source_ref
         )
 
@@ -357,7 +456,7 @@ class StatementSetLocals(StatementChildrenHavingBase):
     def mayRaiseException(self, exception_type):
         return self.getNewLocals().mayRaiseException(exception_type)
 
-    getNewLocals = StatementChildrenHavingBase.childGetter("new_locals")
+    getNewLocals = StatementChildHavingBase.childGetter("new_locals")
 
     def computeStatement(self, trace_collection):
         trace_collection.onExpression(self.getNewLocals())
@@ -375,13 +474,11 @@ class StatementSetLocals(StatementChildrenHavingBase):
             return result, "new_raise", """\
 Setting locals already raises implicitly building new locals."""
 
-        trace_collection.setLocalsDictShape(
-            locals_scope      = self.locals_scope,
-            locals_dict_shape = new_locals.getTypeShape()
-        )
-
         return self, None, None
 
+    def getStatementNiceName(self):
+        return "locals mapping init statement"
+
 
 class StatementSetLocalsDictionary(StatementSetLocals):
     kind = "STATEMENT_SET_LOCALS_DICTIONARY"
@@ -399,14 +496,17 @@ class StatementSetLocalsDictionary(StatementSetLocals):
     def mayRaiseException(self, exception_type):
         return False
 
+    def getStatementNiceName(self):
+        return "locals dictionary init statement"
 
-class StatementReleaseLocals(NodeBase):
+
+class StatementReleaseLocals(StatementBase):
     kind = "STATEMENT_RELEASE_LOCALS"
 
     __slots__ = ("locals_scope",)
 
     def __init__(self, locals_scope, source_ref):
-        NodeBase.__init__(
+        StatementBase.__init__(
             self,
             source_ref = source_ref
         )
@@ -414,11 +514,6 @@ class StatementReleaseLocals(NodeBase):
         self.locals_scope = locals_scope
 
     def computeStatement(self, trace_collection):
-        trace_collection.setLocalsDictShape(
-            locals_scope      = None,
-            locals_dict_shape = None
-        )
-
         return self, None, None
 
     def getDetails(self):
@@ -431,3 +526,6 @@ class StatementReleaseLocals(NodeBase):
 
     def mayRaiseException(self, exception_type):
         return False
+
+    def getStatementNiceName(self):
+        return "locals dictionary release statement"
