@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -162,7 +162,7 @@ def makeRaiseTypeErrorExceptionReplacementFromTemplateAndValue(template,
     if type_name is not None:
         result = makeRaiseExceptionReplacementExpressionFromInstance(
             expression = original_node,
-            exception  = TypeError(template % type_name)
+            exception  = TypeError(template % type_name if '%' in template else template)
         )
 
         result = wrapExpressionWithNodeSideEffects(
