@@ -213,9 +213,6 @@ class ExpressionOutlineFunctionBodyBase(ExpressionFunctionBodyBase):
 
         self.temp_scope = None
 
-    getBody = ExpressionFunctionBodyBase.childGetter("body")
-    setBody = ExpressionFunctionBodyBase.childSetter("body")
-
     def isExpressionOutlineFunctionBodyBase(self):
         return True
 
@@ -338,15 +335,16 @@ class ExpressionOutlineFunctionBodyBase(ExpressionFunctionBodyBase):
         )
 
 
+# TODO: No other uses of the base class, maybe ExpressionOutlineBody should be
+# moved to be so, or that level should be removed.
 class ExpressionOutlineFunction(ExpressionOutlineFunctionBodyBase):
     kind = "EXPRESSION_OUTLINE_FUNCTION"
 
-    named_children = (
-        "body",
-    )
-
     def isEarlyClosure(self):
         return self.provider.isEarlyClosure()
 
     def isUnoptimized(self):
         return self.provider.isUnoptimized()
+
+    def getLocalsScope(self):
+        return self.provider.getLocalsScope()
