@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -35,36 +35,49 @@ from nuitka.importing.ImportCache import (
 )
 from nuitka.ModuleRegistry import addUsedModule
 from nuitka.nodes.NodeMakingHelpers import getComputationResult
+from nuitka.nodes.shapes.BuiltinTypeShapes import ShapeTypeDict
 from nuitka.PythonVersions import python_version
+from nuitka.tree.SourceReading import readSourceLine
 from nuitka.utils.InstanceCounters import counted_del, counted_init
 
-from .VariableTraces import (
-    VariableTraceAssign,
-    VariableTraceInit,
-    VariableTraceLoopMerge,
-    VariableTraceMerge,
-    VariableTraceUninit,
-    VariableTraceUnknown
+from .ValueTraces import (
+    ValueTraceAssign,
+    ValueTraceInit,
+    ValueTraceLoopMerge,
+    ValueTraceMerge,
+    ValueTraceUninit,
+    ValueTraceUnknown
 )
 
 signalChange = None
 
 
 class CollectionTracingMixin(object):
+    """ This contains for logic for maintaining active traces.
+
+        They are kept for "variable" and versions.
+    """
+
     def __init__(self):
-        # For functions, when we are in here, the currently active one,
+        # Currently active values in the tracing.
         self.variable_actives = {}
 
     def getVariableCurrentTrace(self, variable):
+        """ Get the current value trace associated to this variable
+
+            It is also created on the fly if necessary. We create them
+            lazy so to keep the tracing branches minimal where possible.
+        """
+
         return self.getVariableTrace(
             variable = variable,
-            version  = self.getCurrentVariableVersion(variable)
+            version  = self._getCurrentVariableVersion(variable)
         )
 
     def markCurrentVariableTrace(self, variable, version):
         self.variable_actives[variable] = version
 
-    def getCurrentVariableVersion(self, variable):
+    def _getCurrentVariableVersion(self, variable):
         try:
             return self.variable_actives[variable]
         except KeyError:
@@ -90,10 +103,8 @@ class CollectionTracingMixin(object):
             self.addVariableTrace(
                 variable = variable,
                 version  = version,
-                trace    = VariableTraceUnknown(
+                trace    = ValueTraceUnknown(
                     owner    = self.owner,
-                    variable = variable,
-                    version  = version,
                     previous = current
                 )
             )
@@ -107,11 +118,7 @@ class CollectionTracingMixin(object):
 
         version = variable.allocateTargetNumber()
 
-        result = VariableTraceLoopMerge(
-            variable = variable,
-            version  = version,
-            previous = current
-        )
+        result = ValueTraceLoopMerge(current)
 
         self.addVariableTrace(
             variable = variable,
@@ -125,10 +132,15 @@ class CollectionTracingMixin(object):
 
     def markActiveVariablesAsUnknown(self):
         for variable in self.getActiveVariables():
+            if variable.isTempVariable():
+                continue
+
             self.markActiveVariableAsUnknown(variable)
 
 
 class CollectionStartpointMixin(object):
+    # Many things are traces, pylint: disable=too-many-instance-attributes
+
     def __init__(self):
         # Variable assignments performed in here, last issued number, only used
         # to determine the next number that should be used for a new assignment.
@@ -143,6 +155,10 @@ class CollectionStartpointMixin(object):
         self.return_collections = None
         self.exception_collections = None
 
+        self.outline_functions = None
+
+        self.locals_scope = None
+
     def getLoopBreakCollections(self):
         return self.break_collections
 
@@ -232,11 +248,7 @@ class CollectionStartpointMixin(object):
     def addVariableMergeMultipleTrace(self, variable, traces):
         version = variable.allocateTargetNumber()
 
-        trace_merge = VariableTraceMerge(
-            variable = variable,
-            version  = version,
-            traces   = traces
-        )
+        trace_merge = ValueTraceMerge(traces)
 
         self.addVariableTrace(variable, version, trace_merge)
 
@@ -258,11 +270,9 @@ class CollectionStartpointMixin(object):
 
         Tracing.printSeparator()
 
-    def _initVariableUnknown(self, variable):
-        trace = VariableTraceUnknown(
+    def initVariableUnknown(self, variable):
+        trace = ValueTraceUnknown(
             owner    = self.owner,
-            variable = variable,
-            version  = 0,
             previous = None
         )
 
@@ -275,11 +285,7 @@ class CollectionStartpointMixin(object):
         return trace
 
     def _initVariableInit(self, variable):
-        trace = VariableTraceInit(
-            owner    = self.owner,
-            variable = variable,
-            version  = 0
-        )
+        trace = ValueTraceInit(self.owner)
 
         self.addVariableTrace(
             variable = variable,
@@ -290,10 +296,8 @@ class CollectionStartpointMixin(object):
         return trace
 
     def _initVariableUninit(self, variable):
-        trace = VariableTraceUninit(
+        trace = ValueTraceUninit(
             owner    = self.owner,
-            variable = variable,
-            version  = 0,
             previous = None
         )
 
@@ -305,8 +309,17 @@ class CollectionStartpointMixin(object):
 
         return trace
 
-    def updateFromCollection(self, old_collection):
-        Variables.updateFromCollection(old_collection, self)
+    def updateVariablesFromCollection(self, old_collection):
+        Variables.updateVariablesFromCollection(old_collection, self)
+
+    @contextlib.contextmanager
+    def makeLocalsDictContext(self, locals_scope):
+        old_locals_scope = self.locals_scope
+        self.locals_scope = locals_scope
+
+        yield
+
+        self.locals_scope = old_locals_scope
 
     @contextlib.contextmanager
     def makeAbortStackContext(self, catch_breaks, catch_continues,
@@ -341,16 +354,42 @@ class CollectionStartpointMixin(object):
         elif variable.isLocalVariable():
             result = self._initVariableUninit(variable)
         elif variable.isModuleVariable():
-            result = self._initVariableUnknown(variable)
+            result = self.initVariableUnknown(variable)
         elif variable.isTempVariable():
             result = self._initVariableUninit(variable)
+        elif variable.isLocalsDictVariable():
+            if variable.getOwner().getTypeShape() is ShapeTypeDict:
+                result = self._initVariableUninit(variable)
+            else:
+                result = self.initVariableUnknown(variable)
         else:
             assert False, variable
 
-        assert result.getVariable() is variable
-
         return result
 
+    def addOutlineFunction(self, outline):
+        if self.outline_functions is None:
+            self.outline_functions = [outline]
+        else:
+            self.outline_functions.append(outline)
+
+    def getOutlineFunctions(self):
+        return self.outline_functions
+
+    def onLocalsDictEscaped(self, locals_scope):
+        # TODO: Limit to the scope.
+        if locals_scope is not None:
+            for variable in locals_scope.variables.values():
+                self.markActiveVariableAsUnknown(variable)
+
+
+        for variable in self.getActiveVariables():
+            if variable.isTempVariable() or variable.isModuleVariable():
+                continue
+
+            self.markActiveVariableAsUnknown(variable)
+
+
 
 class TraceCollectionBase(CollectionTracingMixin):
     __del__ = counted_del()
@@ -367,7 +406,7 @@ class TraceCollectionBase(CollectionTracingMixin):
         self.value_states = {}
 
     def __repr__(self):
-        return "<%s for %s %d>" % (
+        return "<%s for %s at 0x%x>" % (
             self.__class__.__name__,
             self.name,
             id(self)
@@ -391,8 +430,9 @@ class TraceCollectionBase(CollectionTracingMixin):
     @staticmethod
     def mustNotAlias(a, b):
         # TODO: not yet really implemented
-        if a.isExpressionConstantDictRef() and b.isExpressionConstantRef():
-            return True
+        if a.isExpressionConstantRef() and b.isExpressionConstantRef():
+            if a.isMutable() or b.isMutable():
+                return True
 
         return False
 
@@ -409,9 +449,8 @@ class TraceCollectionBase(CollectionTracingMixin):
 
                 self.markActiveVariableAsUnknown(variable)
 
-            elif python_version >= 300 or variable.isSharedTechnically() is not False:
-                # print variable
-
+            elif variable.isLocalVariable() and \
+                (python_version >= 300 or variable.isSharedTechnically() is not False):
                 # TODO: Could be limited to shared variables that are actually
                 # written to. Most of the time, that won't be the case.
 
@@ -432,17 +471,10 @@ class TraceCollectionBase(CollectionTracingMixin):
     def addVariableMergeMultipleTrace(self, variable, traces):
         return self.parent.addVariableMergeMultipleTrace(variable, traces)
 
-    def onVariableSet(self, assign_node):
-        variable_ref = assign_node.getTargetVariableRef()
-
-        version = variable_ref.getVariableVersion()
-        variable = variable_ref.getVariable()
-
-        variable_trace = VariableTraceAssign(
+    def onVariableSet(self, variable, version, assign_node):
+        variable_trace = ValueTraceAssign(
             owner       = self.owner,
             assign_node = assign_node,
-            variable    = variable,
-            version     = version,
             previous    = self.getVariableCurrentTrace(
                 variable = variable
             )
@@ -459,18 +491,13 @@ class TraceCollectionBase(CollectionTracingMixin):
 
         return variable_trace
 
-    def onVariableDel(self, variable_ref):
+    def onVariableDel(self, variable, version):
         # Add a new trace, allocating a new version for the variable, and
         # remember the delete of the current
-        variable = variable_ref.getVariable()
-        version = variable_ref.getVariableVersion()
-
         old_trace = self.getVariableCurrentTrace(variable)
 
-        variable_trace = VariableTraceUninit(
+        variable_trace = ValueTraceUninit(
             owner    = self.owner,
-            variable = variable,
-            version  = version,
             previous = old_trace
         )
 
@@ -484,47 +511,35 @@ class TraceCollectionBase(CollectionTracingMixin):
         # Make references point to it.
         self.markCurrentVariableTrace(variable, version)
 
+        return variable_trace
+
+    def onLocalsUsage(self, locals_owner):
+        self.onLocalsDictEscaped(locals_owner.getLocalsScope())
 
-    def onLocalsUsage(self):
         result = []
 
-        if self.owner.isExpressionFunctionBody():
-            include_closure = not self.owner.isUnoptimized()
-        else:
-            include_closure = False
+        include_closure = locals_owner.isExpressionFunctionBody() and \
+                          not locals_owner.isUnoptimized()
 
         for variable in self.getActiveVariables():
-
-            # TODO: Currently this is a bit difficult to express in a positive
-            # way, but we want to have only local variables.
-            if not variable.isTempVariable() and \
-               not variable.isModuleVariable() and \
-               (variable.getOwner() is self.owner or include_closure) and \
+            if variable.isLocalVariable() and \
+               (variable.getOwner() is locals_owner or
+                include_closure and locals_owner.hasClosureVariable(variable)) and \
                variable.getName() != ".0":
                 variable_trace = self.getVariableCurrentTrace(
                     variable
                 )
 
                 variable_trace.addNameUsage()
-
                 result.append(
                     (
                         variable,
-                        variable_trace.getVersion()
+                        variable_trace
                     )
                 )
 
         return result
 
-    def onVariableRelease(self, variable):
-        current = self.getVariableCurrentTrace(variable)
-
-        # Annotate that releases to the trace, it may be important knowledge.
-        current.addRelease()
-
-        return current
-
-
     def onVariableContentEscapes(self, variable):
         self.getVariableCurrentTrace(variable).onValueEscape()
 
@@ -556,12 +571,6 @@ class TraceCollectionBase(CollectionTracingMixin):
         if new_node is not expression:
             expression.replaceWith(new_node)
 
-            if new_node.isExpressionVariableRef() or \
-               new_node.isExpressionTempVariableRef():
-                # Remember the reference for constraint collection.
-                assert new_node.variable_trace.hasDefiniteUsages()
-
-
         return new_node
 
     def onStatement(self, statement):
@@ -582,8 +591,10 @@ class TraceCollectionBase(CollectionTracingMixin):
             return new_statement
         except Exception:
             Tracing.printError(
-                "Problem with statement at %s:" %
-                statement.getSourceReference().getAsString()
+                "Problem with statement at %s:\n-> %s" % (
+                    statement.source_ref.getAsString(),
+                    readSourceLine(statement.source_ref)
+                )
             )
             raise
 
@@ -708,6 +719,12 @@ class TraceCollectionBase(CollectionTracingMixin):
             catch_exceptions = catch_exceptions
         )
 
+    def makeLocalsDictContext(self, locals_scope):
+        return self.parent.makeLocalsDictContext(locals_scope)
+
+    def onLocalsDictEscaped(self, locals_scope):
+        self.parent.onLocalsDictEscaped(locals_scope)
+
     def getCompileTimeComputationResult(self, node, computation, description):
         new_node, change_tags, message = getComputationResult(
             node        = node,
@@ -735,6 +752,9 @@ class TraceCollectionBase(CollectionTracingMixin):
         for key in self.value_states:
             self.value_states[key] = None
 
+    def addOutlineFunction(self, outline):
+        self.parent.addOutlineFunction(outline)
+
 
 class TraceCollectionBranch(TraceCollectionBase):
     def __init__(self, name, parent):
@@ -762,12 +782,14 @@ class TraceCollectionBranch(TraceCollectionBase):
 
     def initVariable(self, variable):
         variable_trace = self.parent.initVariable(variable)
-        assert variable_trace.getVersion() == 0
 
         self.variable_actives[variable] = 0
 
         return variable_trace
 
+    def onLocalsDictEscaped(self, locals_scope):
+        return self.parent.onLocalsDictEscaped(locals_scope)
+
     def dumpTraces(self):
         Tracing.printSeparator()
         self.parent.dumpTraces()
@@ -786,7 +808,6 @@ class TraceCollectionFunction(CollectionStartpointMixin,
                               TraceCollectionBase):
     def __init__(self, parent, function_body):
         assert function_body.isExpressionFunctionBody() or \
-               function_body.isExpressionClassBody() or \
                function_body.isExpressionGeneratorObjectBody() or \
                function_body.isExpressionCoroutineObjectBody() or \
                function_body.isExpressionAsyncgenObjectBody(), function_body
@@ -796,7 +817,7 @@ class TraceCollectionFunction(CollectionStartpointMixin,
         TraceCollectionBase.__init__(
             self,
             owner  = function_body,
-            name   = "function_" + str(function_body),
+            name   = "collection_" + function_body.getCodeName(),
             parent = parent
         )
 
@@ -806,19 +827,26 @@ class TraceCollectionFunction(CollectionStartpointMixin,
                 self.variable_actives[parameter_variable] = 0
 
         for closure_variable in function_body.getClosureVariables():
-            self._initVariableUnknown(closure_variable)
+            self.initVariableUnknown(closure_variable)
             self.variable_actives[closure_variable] = 0
 
+        # TODO: Have special function type for exec functions stuff.
+        if function_body.locals_scope is not None:
+            for locals_dict_variable in function_body.locals_scope.variables.values():
+                self._initVariableUninit(locals_dict_variable)
+
 
 class TraceCollectionModule(CollectionStartpointMixin,
                             TraceCollectionBase):
     def __init__(self, module):
+        assert module.isCompiledPythonModule(), module
+
         CollectionStartpointMixin.__init__(self)
 
         TraceCollectionBase.__init__(
             self,
             owner  = module,
-            name   = "module",
+            name   = "module:" + module.getFullName(),
             parent = None
         )
 
