@@ -59,7 +59,6 @@ from nuitka.nodes.FunctionNodes import (
 )
 from nuitka.nodes.LoopNodes import StatementLoop, StatementLoopBreak
 from nuitka.nodes.OperatorNodes import makeBinaryOperationNode
-from nuitka.nodes.ParameterSpecs import ParameterSpec
 from nuitka.nodes.ReturnNodes import StatementReturn
 from nuitka.nodes.SubscriptNodes import (
     ExpressionSubscriptLookup,
@@ -77,6 +76,7 @@ from nuitka.PythonVersions import (
     getComplexCallSequenceErrorTemplate,
     python_version
 )
+from nuitka.specs.ParameterSpecs import ParameterSpec
 
 from .InternalModule import (
     getInternalModule,
